﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace MediaPlayer_vs_AudioGraph_Demo
{

    class CarrierSampleSet
    {
        public static readonly uint SampleRate = 44100;

        public byte[] Samples
        {
            get; private set;
        }

        public TimeSpan Duration
        {
            get; private set;
        }

        public CarrierSampleSet(byte[] samples = null)
        {
            Samples = samples;
            Samples ??= new byte[] { 0xfb, 0x05, 0x80 };
            Duration = TimeSpan.FromSeconds(Samples.Length * 1.0 / SampleRate);
        }
    }

    abstract class WaveformBuilder
    {
        readonly CarrierSampleSet css = new();

        protected abstract void WriteBytes(byte[] bytes);

        public void WriteLeadCode()
        {
            var repeats1 = (int)Math.Round(
                TimeSpan.FromMilliseconds(9) / css.Duration);
            for (var i = 0; i < repeats1; i++)
            {
                WriteBytes(css.Samples);
            }

            var repeats0 = (int)Math.Round(
                TimeSpan.FromMilliseconds(4.5) / (css.Duration / css.Samples.Length));
            for (var i = 0; i < repeats0; i++)
            {
                WriteBytes(new byte[] { 0x80 });
            }
        }

        public void WriteRepeatCode()
        {
            var repeats1 = (int)Math.Round(
                TimeSpan.FromMilliseconds(9) / css.Duration);
            for (var i = 0; i < repeats1; i++)
            {
                WriteBytes(css.Samples);
            }

            var repeats0 = (int)Math.Round(
                TimeSpan.FromMilliseconds(2.25) / (css.Duration / css.Samples.Length));
            for (var i = 0; i < repeats0; i++)
            {
                WriteBytes(new byte[] { 0x80 });
            }

            repeats1 = (int)Math.Round(
                TimeSpan.FromMilliseconds(0.55) / css.Duration);
            for (var i = 0; i < repeats1; i++)
            {
                WriteBytes(css.Samples);
            }
        }

        public void WriteBitCode(bool bit1)
        {
            var repeats1 = (int)Math.Round(
                TimeSpan.FromMilliseconds(0.56) / css.Duration);
            for (var i = 0; i < repeats1; i++)
            {
                WriteBytes(css.Samples);
            }

            var bitDuration = bit1 ? 2.24 : 1.12;
            var repeats0 = (int)Math.Round(
                TimeSpan.FromMilliseconds(bitDuration - 0.56) / (css.Duration / css.Samples.Length));
            for (var i = 0; i < repeats0; i++)
            {
                WriteBytes(new byte[] { 0x80 });
            }
        }

        public void WriteByteCode(byte byte2write)
        {
            for (var i = 0; i < 8; i++)
            {
                WriteBitCode((byte2write >> i & 0x01) != 0);
            }
        }

        public void WritePauseCode(double millileconds)
        {
            var repeats0 = (int)Math.Round(
                TimeSpan.FromMilliseconds(millileconds) / (css.Duration / css.Samples.Length));
            for (var i = 0; i < repeats0; i++)
            {
                WriteBytes(new byte[] { 0x80 });
            }
        }
    }

    interface ISignalOutput
    {
        public bool RepeatFrames { get; set; }
        public bool Repeat1stFrame { get; set; }

        public void StartTransmit(byte lowCustom, byte highCustom, byte data);
    }

}
